'use strict';

postLensApp.factory('wikiARService', function($rootScope, $navigate,
                                              cordovaReady,
                                              selected_shop_DataModel,
                                              shopInfoService) {


	var app = {

	    // represents the device capability of launching ARchitect Worlds with specific features
	    isDeviceSupported: false,

	    // Application Constructor
	    initialize: function() {
	        this.bindEvents();
	    },
	    // Bind Event Listeners
	    //
	    // Bind any events that are required on startup. Common events are:
	    // 'load', 'deviceready', 'offline', and 'online'.
	    bindEvents: function() {
	        document.addEventListener('deviceready', this.onDeviceReady, false);
	    },
	    // deviceready Event Handler
	    onDeviceReady: function() {
	        app.wikitudePlugin = cordova.require("com.wikitude.phonegap.WikitudePlugin.WikitudePlugin");
	        app.wikitudePlugin.isDeviceSupported(app.onDeviceSupportedCallback, app.onDeviceNotSupportedCallback, ["geo"]);
	    },
	    onDeviceSupportedCallback: function() {
	    	app.isDeviceSupported = true;
	    	console.log('device support Wikitude');
	    	$rootScope.$broadcast('wikitudeOK');
	    },
	    onDeviceNotSupportedCallback: function() {
	    	app.isDeviceSupported = false;
	    },
		// This function extracts an url parameter
		getUrlParameterForKey: function(url, requestedParam) {
		    requestedParam = requestedParam.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
		    var regexS = "[\\?&]" + requestedParam + "=([^&#]*)";
		    var regex = new RegExp(regexS);
		    var results = regex.exec(url);

		    if (results == null)
		        return "";
		    else {
		        var result = decodeURIComponent(results[1]);
		        return result;
		    }
		},
	    // --- Wikitude Plugin ---
	    // Use this method to load a specific ARchitect World from either the local file system or a remote server
	    loadARchitectWorld: function(example) {
	    	if (!app.wikitudePlugin) {
	    		app.wikitudePlugin = cordova.require("com.wikitude.phonegap.WikitudePlugin.WikitudePlugin");
			}
	    	if (!example) {
	    		example = {
		    		"requiredFeatures": [
		                // "2d_tracking",
		                "geo"
		            ],
		            "startupConfiguration": {
		                "camera_position": "back"
		            }
	    		};
                if (navigator.userAgent.toLowerCase().match(/android/)) {
                    example.path = "assets/www/ar_layer4/index.html";
    		    } else if (navigator.userAgent.toLowerCase().match(/ipad/)) {
    		    	// example.path = "www/ar_layer/arView.html";
    		    	example.path = "www/ar_layer4/index.html";
    		    } else if (navigator.userAgent.toLowerCase().match(/iphone/)) {
                    // example.path = "www/ar_layer/arView_phone.html";
                    example.path = "www/ar_layer4/index.html";
                } else {
                	console.warn('not supported platform');
                	return;
                }
	    	}
	        // check if the current device is able to launch ARchitect Worlds
	        app.wikitudePlugin.isDeviceSupported(function() {
	            app.wikitudePlugin.setOnUrlInvokeCallback(app.onUrlInvoke);
	            // inject poi data using phonegap's GeoLocation API and inject data using World.loadPoisFromJsonData
	            var onLocationUpdated = function(argument) {
	            	if (argument.coords) {
	            		var c = argument.coords;
	            		app.wikitudePlugin.setLocation(c.latitude, c.longitude, c.altitude, c.accuracy);
	            	}
	            };
	            var onLocationError = function(argument) {

	            };
	            if ( example.requiredExtension === "ObtainPoiDataFromApplicationModel" ) {
	                navigator.geolocation.getCurrentPosition(onLocationUpdated, onLocationError);
	            }

	            console.log('will load AR');
	            app.wikitudePlugin.loadARchitectWorld(function successFn(loadedURL) {
	                /* Respond to successful world loading if you need to */ 
	            }, function errorFn(error) {
	                alert('Loading AR web view failed: ' + error);
	            },
	            example.path, example.requiredFeatures, example.startupConfiguration
	            );
	        }, function(errorMessage) {
	            // alert(errorMessage);
	            var customErr = errorMessage.replace('This device is missing following features',
	            	'อุปกรณ์ของท่านไม่สามารถใช้งาน AR View ได้ เนื่องจากอุปกรณ์ไม่รองรับสิ่งต่อไปนี้');
	            customErr = customErr.replace('Compass', 'Compass (เข็มทิศดิจิทัล)');
	            alert(customErr);
	        },
	        example.requiredFeatures
	        );
	    },
	    urlLauncher: function(url) {
	        var world = {
	            "path": url, 
	            "requiredFeatures": [
	                // "2d_tracking",
	                "geo"
	            ],
	            "startupConfiguration": {
	                "camera_position": "back"
	            }
	        };
	        app.loadARchitectWorld(world);
	    },
	    // This function gets called if you call "document.location = architectsdk://" in your ARchitect World
	    onUrlInvoke: function (url) {
	        if (url.indexOf('captureScreen') > -1) {
	            app.wikitudePlugin.captureScreen(
	                function(absoluteFilePath) {
	                    alert("snapshot stored at:\n" + absoluteFilePath);
	                }, 
	                function (errorMessage) {
	                    alert(errorMessage);                
	                },
	                true, null
	            );
	        } else if ( app.getUrlParameterForKey(url, 'text') ) {
                console.log('onClickInARchitectWorld:getUrlParameterForKey();'+ app.getUrlParameterForKey(url, 'text'));
                app.wikitudePlugin.close();
                //Search ShopInfo by shop_id
                try{
                    var searchCriteria = {};
                    var shopId = app.getUrlParameterForKey(url, 'text');
                    $navigate.go('/branch/'+shopId+'/home');
                    // searchCriteria["shopId"] = shopId;
                    // shopInfoService.searchShop(
                    //     searchCriteria
                    //     , function (r) {
                    //         if (r.error || !r.result){
                    //             console.log('call shopInfoService.searchShop Error');
                    //         }else if (r.result) {
                    //             console.log('shopInfoService.searchShop result: ' + JSON.stringify(r.result));
                    //             selected_shop_DataModel.selectedShop = r.result.shopResults[0];
                    //             $rootScope.$apply(function () {
                    //                 if(selected_shop_DataModel.selectedShop.shopType=='S'){
                    //                     $navigate.go('/shop', 'slide');
                    //                 }else{
                    //                     $navigate.go('/branch', 'slide');
                    //                 }
                    //             });
                    //         }
                    //     });
                }catch(err){
                    console.log('Error:navigate:'+err);alert('Can\'t display location information!');
                }
//	                try{
//	                    $rootScope.$apply(function () {
//	                        $navigate.go('/track', 'slide');
//	                    });
//	                }catch(err){console.log('Error:navigate:'+err);alert('erererererer');}
//	                app.report( "you clicked on a label with text: " + app.getUrlParameterForKey(url, 'text') );
            } else if ( app.getUrlParameterForKey(url, 'action') ) {

                app.wikitudePlugin.close();

            } else if ( app.getUrlParameterForKey(url, 'status') ) {
                app.wikitudePlugin.hide();
	            
	        } else {
	            alert(url + "not supported");
	        }
	    }
	    // --- End Wikitude Plugin ---
	};


/*
	var app = {

		    isDeviceSupported: false,

		    // Application Constructor
		    initialize: function() {
		    	console.log('wikiARService: initialize');
		        this.bindEvents();
		    },
		    // Bind Event Listeners
		    //
		    // Bind any events that are required on startup. Common events are:
		    // 'load', 'deviceready', 'offline', and 'online'.
		    bindEvents: function() {
		        document.addEventListener('deviceready', this.onDeviceReady, false);
		    },
		    // deviceready Event Handler
		    //
		    // The scope of 'this' is the event. In order to call the 'receivedEvent'
		    // function, we must explicity call 'app.receivedEvent(...);'
		    onDeviceReady: function() {
		    	console.log('wikiARService.onDeviceReady()');

		        // check if the current device is able to launch ARchitect Worlds
		        app.wikitudePlugin = cordova.require("com.wikitude.phonegap.WikitudePlugin.WikitudePlugin");
		        app.wikitudePlugin.isDeviceSupported(app.onDeviceSupportedCallback, app.onDeviceNotSupportedCallback);
		    },
		    // This function extracts an url parameter
		    getUrlParameterForKey: function(url, requestedParam) {
		        requestedParam = requestedParam.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
		        var regexS = "[\\?&]" + requestedParam + "=([^&#]*)";
		        var regex = new RegExp(regexS);
		        var results = regex.exec(url);

		        if (results == null)
		            return "";
		        else {
		            var result = decodeURIComponent(results[1]);
		            return result;
		        }
		    },
		    // --- Wikitude Plugin ---
		    // A callback which gets called if the device is able to launch ARchitect Worlds
		    onDeviceSupportedCallback: function() {
		        app.isDeviceSupported = true;
		    },

		    // A callback which gets called if the device is not able to start ARchitect Worlds
		    onDeviceNotSupportedCallback: function() {
		        alert('Unable to launch ARchitect Worlds on this device');
		    },
		    // Use this method to load a specific ARchitect World from either the local file system or a remote server
		    loadARchitectWorld: function() {
		    	if (!app.wikitudePlugin) {
		    		console.warn('uninitialized wikitude');
		    		return;
		    	}

		    	console.log('wikiARService.loadARchitectWorld()');
		        app.wikitudePlugin.setOnUrlInvokeCallback(app.onUrlInvoke);

		        var successFn = function(loadedURL) {
		        	console.log('load ar successfully', loadedURL)
		        };
		        var failFn = function(error) {
		        	console.error('load ar fail', error)
		        };

		        if (app.isDeviceSupported) {
	                if (navigator.userAgent.toLowerCase().match(/android/)) {
	                    app.wikitudePlugin.loadARchitectWorld("assets/www/ar_layer4/index.html", app.defaultRequiredFeatures, app.defaultStartupConfiguration);
	    		    } else if (navigator.userAgent.toLowerCase().match(/ipad/)) {
	    		    	app.wikitudePlugin.loadARchitectWorld(successFn, failFn, "www/ar_layer/arView.html", app.defaultRequiredFeatures, app.defaultStartupConfiguration);
	    		    } else if (navigator.userAgent.toLowerCase().match(/iphone/)) {
	                    app.wikitudePlugin.loadARchitectWorld(successFn, failFn, "www/ar_layer/arView_phone.html", app.defaultRequiredFeatures, app.defaultStartupConfiguration);
	                }
		        } else {
		            alert("Device is not supported");
		        }
		    },
		    // called when the screen was captured successfully
		    onScreenCaptured: function (absoluteFilePath) {
		        alert("snapshot stored at:\n" + absoluteFilePath);
		    },
		    // called when a screen capture error occurs
		    onScreenCapturedError: function (errorMessage) {
		        alert(errorMessage);
		    },
		    // This function gets called if you call "document.location = architectsdk://" in your ARchitect World
		    onUrlInvoke: function (url) {
		        if (url.indexOf('captureScreen') > -1) {
		            app.wikitudePlugin.captureScreen(true, null, app.onScreenCaptured, app.onScreenCapturedError);
		        } else if ( app.getUrlParameterForKey(url, 'text') ) {
	                console.log('onClickInARchitectWorld:getUrlParameterForKey();'+ app.getUrlParameterForKey(url, 'text'));
	                app.wikitudePlugin.close();
	                //Search ShopInfo by shop_id
	                try{
	                    var searchCriteria = {};
	                    searchCriteria["shopId"] = app.getUrlParameterForKey(url, 'text');
	                    shopInfoService.searchShop(
	                        searchCriteria
	                        , function (r) {
	                            if (r.error || !r.result){
	                                console.log('call shopInfoService.searchShop Error');
	                            }else if (r.result) {
	                                console.log('shopInfoService.searchShop result: ' + JSON.stringify(r.result));
	                                selected_shop_DataModel.selectedShop = r.result.shopResults[0];
	                                $rootScope.$apply(function () {
	                                    if(selected_shop_DataModel.selectedShop.shopType=='S'){
	                                        $navigate.go('/shop', 'slide');
	                                    }else{
	                                        $navigate.go('/branch', 'slide');
	                                    }
	                                });
	                            }
	                        });
	                }catch(err){
	                    console.log('Error:navigate:'+err);alert('Can\'t display location information!');
	                }
//	                try{
//	                    $rootScope.$apply(function () {
//	                        $navigate.go('/track', 'slide');
//	                    });
//	                }catch(err){console.log('Error:navigate:'+err);alert('erererererer');}
//	                app.report( "you clicked on a label with text: " + app.getUrlParameterForKey(url, 'text') );
	            } else if ( app.getUrlParameterForKey(url, 'action') ) {

	                app.wikitudePlugin.close();

	            } else if ( app.getUrlParameterForKey(url, 'status') ) {
	                app.wikitudePlugin.hide();
		            
		        } else {
		            alert(url + "not supported");
		        }
		    }
		    // --- End Wikitude Plugin ---
		}; */
    return app;
});