'use strict';

postLensApp.factory('geolocation', function ($rootScope, cordovaReady, uiGmapGoogleMapApi) {
	var directionsDisplay;
	var startMarker;
	var endMarker;
	var markers;

	var gmapInstance;
	uiGmapGoogleMapApi.then(function(_map){
		gmapInstance = _map;
	})
	
	var setStartMarker = function(myLatLng, map, markerOpt){
		if(startMarker){
			startMarker.setMap(null);
		}
		
		markerOpt.position = myLatLng;
		markerOpt.map = map;
		startMarker = new gmapInstance.Marker(markerOpt);
	}
	
	var setEndMarker = function(myLatLng, map, markerOpt){
		if(endMarker){
			endMarker.setMap(null);
		}
		
		markerOpt.position = myLatLng;
		markerOpt.map = map;
		endMarker = new gmapInstance.Marker(markerOpt);
	}
	
	var clearMap = function(){
		if(directionsDisplay){
			directionsDisplay.setMap(null);
			directionsDisplay.setPanel(null);
		}
		if(markers){
			for(var i = 0;i < markers.length;++i){
				markers[i].setMap(null);
			}
		}
	}
	
	  return {
		 getCurrentPosition: function (onSuccess, onError, options) {
            navigator.geolocation.getCurrentPosition(
                function () {
                    var that = this,
                      args = arguments;

                    console.log("getCurrentPosition onsuccess geo...");

                    if (onSuccess) {
                          $rootScope.$apply(function () {
                            onSuccess.apply(that, args);
                          });
                    }
	            },
                function () {
                    var that = this,
                      args = arguments;

                    console.log("getCurrentPosition onError geo...");

                    if (onError) {
                      $rootScope.$apply(function () {
                        onError.apply(that, args);
                      });
                    }
	            },
	            options);
	    }
	  
	    ,getCurrentPositionOnload: cordovaReady(function (onSuccess, onError, options) {
		      navigator.geolocation.getCurrentPosition(function () {
			        var that = this,
			          args = arguments;
			        
			        console.log("getCurrentPositionOnload onsuccess geo...");
			        
			        if (onSuccess) {
//                        if(!$scope.$$phase) {
                          $rootScope.$apply(function () {
                            onSuccess.apply(that, args);
                          });
//                        }else{
//                            onSuccess.apply(that, args);
//                        }
			        }
			      }, function () {
			        var that = this,
			          args = arguments;
			          
			        console.log("getCurrentPositionOnload onError geo...");
			        
			        if (onError) {
			          $rootScope.$apply(function () {
			            onError.apply(that, args);
			          });
			        }
			      },
			      options);
		})
		
		,getDirection: function(scope, startLat, startLong, endLat, endLong
				, map, directionsServiceOpt, directionsRendererOpt, routeOpt
				, startMarkerOpt, endMarkerOpt
				, isShowDirectionStep){
			
			clearMap();

			var directionsService = new gmapInstance.DirectionsService(directionsServiceOpt);
			directionsDisplay = new gmapInstance.DirectionsRenderer(directionsRendererOpt);
			var start = new gmapInstance.LatLng(startLat,startLong);
			var end = new gmapInstance.LatLng(endLat,endLong);
			markers = [];
			
			routeOpt.origin = start;
			routeOpt.destination = end;
			
			if(isShowDirectionStep){
//	    		console.log("Show Direction Step...");
	    		scope.$broadcast('showDirection', directionsDisplay);
	    	}
			
			directionsService.route(routeOpt, function(result, status) {
			    if (status == gmapInstance.DirectionsStatus.OK) {
			    	directionsDisplay.setMap(map);
			    	
			    	console.log("Show Direction Map...");
		    		directionsDisplay.setDirections(result);

//                    console.log("Show Direction Map result:"+result);
				    var myRoute = result.routes[0].legs[0];
				    
				    setStartMarker(myRoute.start_location, map, startMarkerOpt);
				    setEndMarker(myRoute.end_location, map, endMarkerOpt);

				    //directionsDisplay.setPanel(document.getElementById("directionsPanel"));
			    	
			    	
				}else if (status == gmapInstance.DirectionsStatus.ZERO_RESULTS){
			    	alert("No direction available");
				}else{
			    	alert("get direction error!");
			    }
			  });
			  
		}
	    
	    ,makeMarker: function (myLatLng, map, image){
			var markerOpt = {
				      position: myLatLng,
				      map: map
			};
			if(image){
				markerOpt.icon = image;
			}
			
			markers.push(new gmapInstance.Marker(markerOpt));
		}
	    
	    ,setStartMarker: setStartMarker
	    ,setEndMarker: setEndMarker
	    ,clearMap: clearMap
	    ,getMarkers: function(){
	    	return markers;
	    }
	    ,setMarkers: function(data){
	    	markers = data;
	    }
        ,watchPosition: function (onSuccess, onError, options) {
              return navigator.geolocation.watchPosition(function () {
                      var that = this,
                          args = arguments;

//                      console.log("watchPosition onSuccess");

                      if (onSuccess) {
                          $rootScope.$apply(function () {
                              onSuccess.apply(that, args);
                          });
                      }
                  }, function () {
                      var that = this,
                          args = arguments;

                      console.log("watchPosition onError");

                      if (onError) {
                          $rootScope.$apply(function () {
                              onError.apply(that, args);
                          });
                      }
                  },
                  options);
          }
        ,clearWatch: function(watchID) {
        	navigator.geolocation.clearWatch(watchID);
        }
        ,calculateDistance: function(lat1, lng1, lat2, lng2) {
        	if (typeof(Number.prototype.toRad) === 'undefined') {
			  Number.prototype.toRad = function() {
			    return this * Math.PI / 180;
			  }
			}
			  var dLat = (lat2-lat1).toRad(),
			      dLng = (lng2-lng1).toRad(),
			      lat1 = lat1.toRad(),
			      lat2 = lat2.toRad();

			  var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
			          Math.sin(dLng / 2) * Math.sin(dLng / 2) * 
			          Math.cos(lat1) * Math.cos(lat2),
			      c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));

			  return 3956 * c; //miles
			  //return 6971 * c //kms
		}
	  };
	});
